using System;
using System.Linq;
using System.Diagnostics;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using System.Collections.ObjectModel;

namespace LoggingWebRole
{
    // Sample Web Role for Azure Diagnostics MSDN article

    public class WebRole : RoleEntryPoint
    {
        public static WebDiagnostics     Diagnostics = new WebDiagnostics();

        // This is  list of configuration settings (in ServiceConfiguration.cscfg) that if changed
        // should not result in the role being recycled.  This is implemented in the RoleEnvironmentChanging
        // and RoleEnvironmentChanged callbacks below
        private static string[] _exemptConfigurationItems = new[] { "ConfigTrace", "WebTrace" };

        public override bool OnStart()
        {
            Diagnostics.WriteDiagnosticInfo(WebRole.Diagnostics.ConfigTrace, TraceEventType.Verbose, WebDiagnostics.TraceEventID.traceFunctionEntry, "Entering WebRole OnStart");

            // Set up for change notifications if any of our configuration values change at run-time.
            // First Changing is called - this determines whether or not the role is recycled based on what
            // is changing.  Then Changed is called to read the new values.
            RoleEnvironment.Changing += RoleEnvironmentChanging;
            RoleEnvironment.Changed += RoleEnvironmentChanged;

            // This code was added to the standard OnStart to trigger transfer of Azure logs
            // to the Diagnostic Storage every one minute for the WebRole.
            // You can change the time interval and
            // level of verbosity of logs (e.g. for only errors) by adjusting the
            // DiagnosticMonitorConfiguration values below.  If you don't want to transfer logs to
            // storage at all, just call DiagnosticMonitor.Start("DiagnosticsConnectionString") without
            // passing the updated DiagnosticMonitorConfiguration structure.

            DiagnosticMonitorConfiguration dmc = DiagnosticMonitor.GetDefaultInitialConfiguration();
            TimeSpan tsOneMinute = TimeSpan.FromMinutes(1);

            // Transfer logs to storage every minute
            dmc.Logs.ScheduledTransferPeriod = tsOneMinute;
            // Transfer verbose, critical, etc. logs
            dmc.Logs.ScheduledTransferLogLevelFilter = LogLevel.Verbose;

            Diagnostics.WriteDiagnosticInfo(WebRole.Diagnostics.ConfigTrace, TraceEventType.Verbose, WebDiagnostics.TraceEventID.traceGeneral, "Setting up Diagnostics Monitor - transfer Verbose logs every minute.");

            // Start up the diagnostic manager with the given configuration.
            try
            {
                DiagnosticMonitor.Start("DiagnosticsConnectionString", dmc);
            }
            catch (Exception exp)
            {
                Diagnostics.WriteDiagnosticInfo(WebRole.Diagnostics.ConfigTrace, TraceEventType.Critical, WebDiagnostics.TraceEventID.traceException, "DiagnosticsManager.Start threw an exception " + exp.ToString());
            }


            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            RoleEnvironment.Changing += RoleEnvironmentChanging;

            return base.OnStart();
        }

        /// <summary>
        /// HasNonExemptConfigurationChanges - Check if config changes contain any that aren't on our "exempt from recycle" list
        /// Returns TRUE if there is at least one config change that isn't on our list.
        /// </summary>
        /// <param name="chgs">Collection of changes from RoleEnvironmentChanging or RoleEnvironmentChanged</param>
        /// <returns></returns>
        private bool HasNonExemptConfigurationChanges(ReadOnlyCollection<RoleEnvironmentChange> chgs)
        {
            Func<RoleEnvironmentConfigurationSettingChange, bool> changeIsNonExempt =
                    x => !_exemptConfigurationItems.Contains(x.ConfigurationSettingName);

            var environmentChanges = chgs.OfType<RoleEnvironmentConfigurationSettingChange>();

            return environmentChanges.Any(changeIsNonExempt);
        }


        /// <summary>
        /// RoleEventChanging - Called when a change is about to be applied to the role.  Determines whether or not to recycle the role instance.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e">A list of what is changing</param>
        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            // Note: e.Cancel == true -> Azure should recycle the role.  If all the changes are in our "exempt" list,
            // we don't need to recycle the role.

            e.Cancel = HasNonExemptConfigurationChanges(e.Changes);

            // Note that we use Trace.WriteLine here rather than going through the Diagnostics class so that we will always log
            // this, even when the switch for whether to log or not is being changed.

            if (!e.Cancel)
            {
                Trace.WriteLine("WebRole::RoleEnvironmentChanging - role is not recycling, getting new switch values from config file.");
            }
            else
            {
                Trace.WriteLine("WebRole::RoleEnvironmentChanging - recycling role instance due to non-exempt configuration changes.");
            }
        }

        /// <summary>
        /// RoleEnvironmentChanged - Called after a change has been applied to the role.  
        /// NOTE: This is called AFTER RoleEnvironmentChanging is called.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e">List of what has changed</param>
        private void RoleEnvironmentChanged(object sender, RoleEnvironmentChangedEventArgs e)
        {
            // Refresh the diagnostic switches from the role config values.
            // This allows for run-time changing of the values of the switches without recycling (i.e. rebooting)
            // the role so we can turn on or off more verbose diagnostic output based on the switches we've
            // defined in ServiceConfiguration.cscfg.
            Diagnostics.GetTraceSwitchValuesFromRoleConfiguration();

            // Log the change to the logs - using Trace.WriteLine to circumvent the switches so that if the switch was
            // turned off or to a low setting like Critical, the message showing this will still go into the logs.
            Trace.WriteLine("WebRole::RoleEnvironmentChanged - Diagnostics switch values changed.  ConfigTrace = " +
                                Diagnostics.ConfigTrace.Switch.Level.ToString() + " WebTrace = " +
                                Diagnostics.WebTrace.Switch.Level.ToString());
        }
    }
}