﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;

namespace LoggingWorkerRole
{
    class WorkerDiagnostics
    {
        // Trace sources - Think of these as multiple output channels. So write to the ConfigTrace when you're 
        // logging information about configuration, write to the WorkerTrace when you're logging information about
        // the worker role activities themselves (e.g. OnStart, Run, etc.).  You can add more sources if you want - they
        // give you more ability to control what is and isn't logged.  For instance, rather than a single WorkerTrace,
        // you might have a source per major component of your app - database, search, etc.
        public TraceSource ConfigTrace;
        public TraceSource WorkerTrace;
        // Add additional sources here

        // Corresponding trace switches to control level of output for each source.  Note that having switches
        // 1:1 with TraceSource's is probably overkill since there is an internal switch in each source, so if
        // you're going to have one switch per source you may as well just use the TraceSource.Switch itself.  
        // (Code in the ctor for this class hooks up the switches below to the corresponding TraceSources so
        // that the internal switch isn't used by the source.)
        // Switches become useful when you have multiple source's sharing a single switch so you can control
        // the output from multiple sources with one change in a switch value.
        public SourceSwitch ConfigTraceSwitch { get; set; }
        public SourceSwitch WorkerTraceSwitch { get; set; }
        // Add additional switches here

        public SourceLevels SourceLevelFromString(string str)
        {
            SourceLevels lvl;

            try
            {
                lvl = (SourceLevels)Enum.Parse(typeof(SourceLevels), str, true);
            }
            catch (System.ArgumentException exp)
            {
                // Invalid value - just default to off.
                lvl = SourceLevels.Off;
            }

            return lvl;
        }

        // Event types
        public enum TraceEventID
        {
            traceGeneral = 0,           // General type
            traceFunctionEntry = 1,
            traceFunctionExit = 2,      // Note that can see just entry / exit by querying eventId lt 3
            traceException = 100,
            traceUnexpected = 200,
            traceFlow = 300             // Can see entry / exit plus errors with eventId le 200
        };

        /// <summary>
        /// Ctor - Set up hooks between sources and switches and read the initial values for the switches
        /// from the ServiceConfiguration.cscfg file.
        /// </summary>
        public WorkerDiagnostics()
        {
            ConfigTraceSwitch = new SourceSwitch("ConfigTrace", "Controls level of output having to do with configuration information");
            WorkerTraceSwitch = new SourceSwitch("WorkerTrace", "Controls level of output having to do with worker role information");
            // Add additional switches here

            // Hook the ConfigTraceSwitch to the ConfigTrace source
            ConfigTrace = new TraceSource("ConfigTrace");
            ConfigTrace.Switch = ConfigTraceSwitch;

            // Hook the WorkerTraceSwitch to the WorkerTrace source
            WorkerTrace = new TraceSource("WorkerTrace");
            WorkerTrace.Switch = WorkerTraceSwitch;

            // Add additional sources here.

            // Now read all the initial values for the switches from the ServiceConfiguration.cscfg file.
            GetTraceSwitchValuesFromRoleConfiguration();
        }

        /// <summary>
        /// GetTraceSwitchValuesFromRoleConfiguration - Read the values for the Source Switches from the 
        /// ServiceConfiguration.cscfg.  Note that this will override anything in app.config - it's to
        /// allow easy changing of these at run-time.
        /// </summary>
        public void GetTraceSwitchValuesFromRoleConfiguration()
        {
            ConfigTraceSwitch.Level = SourceLevelFromString(RoleEnvironment.GetConfigurationSettingValue("ConfigTrace"));
            WorkerTraceSwitch.Level = SourceLevelFromString(RoleEnvironment.GetConfigurationSettingValue("WorkerTrace"));
            // Uses Trace.WriteLine so that the information goes through regardless of the switch values
            Trace.WriteLine("GetTraceSwitchValuesFromRoleConfiguration - Trace switch values set:  Config="+ConfigTraceSwitch.Level.ToString()+
                "  Worker=" + WorkerTraceSwitch.Level.ToString() +
                " Source Switches: Config=" + ConfigTrace.Switch.Level.ToString() +
                " Worker=" + WorkerTrace.Switch.Level.ToString());
        }


        public void WriteDiagnosticInfo(TraceSource src, TraceEventType evtType, TraceEventID evtID, string msg)
        {
            if (src != null)
                src.TraceEvent(evtType, (int)evtID, msg);
        }

        public void WriteDiagnosticInfo(TraceSource src, TraceEventType evtType, TraceEventID evtID, string format, params object[] args)
        {
            if (src != null)
                src.TraceEvent(evtType, (int)evtID, format, args);
        }
    }
}
