﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using System.Collections.ObjectModel;

namespace LoggingWorkerRole
{
    public class WorkerRole : RoleEntryPoint
    {
        // Class to handle diagnostic output.
        private static WorkerDiagnostics     Diagnostics = new WorkerDiagnostics();

        // This is  list of configuration settings (in ServiceConfiguration.cscfg) that if changed
        // should not result in the role being recycled.  This is implemented in the RoleEnvironmentChanging
        // and RoleEnvironmentChanged callbacks below
        private static string[] _exemptConfigurationItems = new[] { "ConfigTrace", "WorkerTrace" };

        /// <summary>
        /// Run - This is the main entry point to the Worker role.  Note that returning from here causes
        /// the role to exit - so you shouldn't return but generally be in a loop looking for work,
        /// processing it, and then waiting for more work. This sample implementation just sleeps for 10
        /// seconds and then logs some sample diagnostic output.
        /// </summary>
        public override void Run()
        {
            while (true)
            {
                Diagnostics.WriteDiagnosticInfo(Diagnostics.WorkerTrace, TraceEventType.Information, WorkerDiagnostics.TraceEventID.traceFlow, "WorkerRole Run - Sleeping 10 seconds");
                Thread.Sleep(10000);
                Diagnostics.WriteDiagnosticInfo(Diagnostics.WorkerTrace, TraceEventType.Information, WorkerDiagnostics.TraceEventID.traceFlow, "WorkerRole Run - Doing work");

                // Here's where you would do actual work.  These are bogus lines just to create different types of messages in the diagnostics store
                // as an example
                Trace.WriteLine("WorkerRole::Run - ConfigTrace = " + Diagnostics.ConfigTrace.Switch.Level.ToString() + " WorkerTrace = " + Diagnostics.WorkerTrace.Switch.Level.ToString());
                Diagnostics.WriteDiagnosticInfo(Diagnostics.WorkerTrace, TraceEventType.Critical, WorkerDiagnostics.TraceEventID.traceGeneral, "Worker Role Run - sample critical message");
                Diagnostics.WriteDiagnosticInfo(Diagnostics.WorkerTrace, TraceEventType.Error, WorkerDiagnostics.TraceEventID.traceGeneral, "Worker Role Run - sample error message");
                Diagnostics.WriteDiagnosticInfo(Diagnostics.WorkerTrace, TraceEventType.Warning, WorkerDiagnostics.TraceEventID.traceGeneral, "Worker Role Run - sample warning message");
                Diagnostics.WriteDiagnosticInfo(Diagnostics.WorkerTrace, TraceEventType.Information, WorkerDiagnostics.TraceEventID.traceGeneral, "Worker Role Run - sample Informational message");
                Diagnostics.WriteDiagnosticInfo(Diagnostics.WorkerTrace, TraceEventType.Verbose, WorkerDiagnostics.TraceEventID.traceGeneral, "Worker Role Run - sample verbose message");
            }
        }

        /// <summary>
        /// OnStart - called when the role starts.  Set up callbacks for configuration changes and
        /// initialize Diagnostic Manager to do transfers to persistent storage regularly.
        /// </summary>
        /// <returns></returns>
        public override bool OnStart()
        {
            Diagnostics.WriteDiagnosticInfo(Diagnostics.WorkerTrace, TraceEventType.Verbose, WorkerDiagnostics.TraceEventID.traceFunctionEntry, "Entering WorkerRole OnStart");

            // Set up for change notifications if any of our configuration values change at run-time.
            // First Changing is called - this determines whether or not the role is recycled based on what
            // is changing.  Then Changed is called to read the new values.
            RoleEnvironment.Changing += RoleEnvironmentChanging;
            RoleEnvironment.Changed += RoleEnvironmentChanged;

            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            // This code was added to the standard OnStart to trigger transfer of Azure logs
            // to the Diagnostic Storage every one minute.  You can change the time interval and
            // level of verbosity of logs (e.g. for only errors) by adjusting the
            // DiagnosticMonitorConfiguration values below.  If you don't want to transfer logs to
            // storage at all, just call DiagnosticMonitor.Start("DiagnosticsConnectionString") without
            // passing the updated DiagnosticMonitorConfiguration structure.

            Diagnostics.WriteDiagnosticInfo(Diagnostics.ConfigTrace, TraceEventType.Verbose, WorkerDiagnostics.TraceEventID.traceGeneral, "Setting up Diagnostics Monitor - transfer Verbose logs every minute.");

            DiagnosticMonitorConfiguration dmc = DiagnosticMonitor.GetDefaultInitialConfiguration();
            TimeSpan tsOneMinute = TimeSpan.FromMinutes(1);

            // Transfer Azure logs to storage every minute
            dmc.Logs.ScheduledTransferPeriod = tsOneMinute;
            // Transfer verbose, critical, etc. logs
            dmc.Logs.ScheduledTransferLogLevelFilter = LogLevel.Verbose;

            // Add System and Application Windows Event Logs to transfer every 10 minutes
            TimeSpan tsTenMinutes = TimeSpan.FromMinutes(10);

            dmc.WindowsEventLog.DataSources.Add("System!*");
            dmc.WindowsEventLog.DataSources.Add("Application!*");
            dmc.WindowsEventLog.ScheduledTransferLogLevelFilter = LogLevel.Verbose;
            dmc.WindowsEventLog.ScheduledTransferPeriod = tsTenMinutes;

            // Set up a performance counter for CPU time
            PerformanceCounterConfiguration pccCPU = new PerformanceCounterConfiguration();
            pccCPU.CounterSpecifier = @"\Processor(_Total)\% Processor Time";
            pccCPU.SampleRate = TimeSpan.FromSeconds(5);

            dmc.PerformanceCounters.ScheduledTransferPeriod = tsTenMinutes;
            dmc.PerformanceCounters.DataSources.Add(pccCPU);

            // Start up the diagnostic manager with the given configuration.
            try
            {
                DiagnosticMonitor.Start("DiagnosticsConnectionString", dmc);
            }
            catch (Exception exp)
            {
                Diagnostics.WriteDiagnosticInfo(Diagnostics.ConfigTrace, TraceEventType.Critical, WorkerDiagnostics.TraceEventID.traceException, "DiagnosticsManager.Start threw an exception " + exp.ToString());
            }

            // Uncomment to enable crash mini dumps for worker role.
            // CrashDumps.EnableCollection(false);

            return base.OnStart();
        }

        /// <summary>
        /// OnStop - Called if the role is being stopped.  Log this.
        /// </summary>
        public override void OnStop()
        {
            Trace.WriteLine("WorkerRole::OnStop - ConfigTrace = " + Diagnostics.ConfigTrace.Switch.Level.ToString() + " WorkerTrace = " + Diagnostics.WorkerTrace.Switch.Level.ToString());
            Diagnostics.WriteDiagnosticInfo(Diagnostics.WorkerTrace, TraceEventType.Verbose, WorkerDiagnostics.TraceEventID.traceFunctionEntry, "Entering WorkerRole OnStop");

            base.OnStop();
        }

        /// <summary>
        /// HasNonExemptConfigurationChanges - Check if config changes contain any that aren't on our "exempt from recycle" list
        /// Returns TRUE if there is at least one config change that isn't on our list.
        /// </summary>
        /// <param name="chgs">Collection of changes from RoleEnvironmentChanging or RoleEnvironmentChanged</param>
        /// <returns></returns>
        private bool HasNonExemptConfigurationChanges(ReadOnlyCollection<RoleEnvironmentChange> chgs)
        {
            Func<RoleEnvironmentConfigurationSettingChange, bool> changeIsNonExempt =
                    x => !_exemptConfigurationItems.Contains(x.ConfigurationSettingName);

            var environmentChanges = chgs.OfType<RoleEnvironmentConfigurationSettingChange>();

            return environmentChanges.Any(changeIsNonExempt);
        }


        /// <summary>
        /// RoleEventChanging - Called when a change is about to be applied to the role.  Determines whether or not to recycle the role instance.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e">A list of what is changing</param>
        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            // Note: e.Cancel == true -> Azure should recycle the role.  If all the changes are in our "exempt" list,
            // we don't need to recycle the role.

            e.Cancel = HasNonExemptConfigurationChanges(e.Changes);

            // Note that we use Trace.WriteLine here rather than going through the Diagnostics class so that we will always log
            // this, even when the switch for whether to log or not is being changed.

            if (!e.Cancel)
            {
                Trace.WriteLine("WorkerRole::RoleEnvironmentChanging - role is not recycling, getting new switch values from config file.");
            }
            else
            {
                Trace.WriteLine("WorkerRole::RoleEnvironmentChanging - recycling role instance due to non-exempt configuration changes.");
            }
        }

        /// <summary>
        /// RoleEnvironmentChanged - Called after a change has been applied to the role.  
        /// NOTE: This is called AFTER RoleEnvironmentChanging is called.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e">List of what has changed</param>
        private void RoleEnvironmentChanged(object sender, RoleEnvironmentChangedEventArgs e)
        {
            // Refresh the diagnostic switches from the role config values.
            // This allows for run-time changing of the values of the switches without recycling (i.e. rebooting)
            // the role so we can turn on or off more verbose diagnostic output based on the switches we've
            // defined in ServiceConfiguration.cscfg.
            Diagnostics.GetTraceSwitchValuesFromRoleConfiguration();

            // Log the change to the logs - using Trace.WriteLine to circumvent the switches so that if the switch was
            // turned off or to a low setting like Critical, the message showing this will still go into the logs.
            Trace.WriteLine("WorkerRole::RoleEnvironmentChanged - Diagnostics switch values changed.  ConfigTrace = " + 
                                Diagnostics.ConfigTrace.Switch.Level.ToString() + " WorkerTrace = " + 
                                Diagnostics.WorkerTrace.Switch.Level.ToString());
        }
    }
}
